#include "WiFi2_Config.h"
#include "HLK_M30_AT.h"

//Pins config
sbit WiFi2_DEF at GPIOA_ODR.B5;
sbit WiFi2_RST at GPIOC_ODR.B3;
sbit WiFi2_TX at GPIOD_ODR.B6;
sbit WiFi2_RX at GPIOD_ODR.B5;


char rx_buff[BUFF_LEN];
char data_ready = 0;
char data_len = 0;

void WiFi2_PowerON() {
  WiFi2_DEF = 1;
  WiFi2_RST = 0;
  WiFi2_TX = 0;
  WiFi2_RX = 0;
  delay_ms(500);
  WiFi2_RST = 1;
  delay_ms(100);
  WiFi2_TX = 1;
  delay_ms(100);
}

void WiFi2_SetCommandMode() {
  WiFi2_DEF = 0;
  delay_ms(1000);
  wiFi2_DEF = 1;
  delay_ms(100);
}

void WiFi2_SetDefaultMode() {                                                   //HW Reset to default
  WiFi2_DEF = 0;
  delay_ms(6000);
  wiFi2_DEF = 1;
}

void WiFi2_Init() {
 GPIO_Digital_Output (&GPIOD_BASE, _GPIO_PINMASK_5 | _GPIO_PINMASK_6);
 GPIO_Digital_Output (&GPIOA_BASE, _GPIO_PINMASK_5);
 GPIO_Digital_Output (&GPIOC_BASE, _GPIO_PINMASK_3);
 WiFi2_PowerON();
 UART2_Init_Advanced(115200, _UART_8_BIT_DATA, _UART_NOPARITY, _UART_ONE_STOPBIT, &_GPIO_MODULE_USART2_PD56);
}

void WiFi2_CMD(const char * AT_CMD, char * VALUE) {
  UART2_Write_Text(AT_CMD);
  UART2_Write_Text(VALUE);
  UART2_Write(0x0D);
}


char WaitToResponse(const char * CMD) {
   char result = 0;
   while(!data_ready);
   data_ready = 0;
   if(strstr(rx_buff,CMD)) {
      result = 1;
   }
   memset(rx_buff,0,data_len);
   data_len = 0;
   return result;
}

//******************************************************************************
//                             CLIENT CONFIG
//******************************************************************************

//******************************************************************************
//Configure WiFi2 click as TCP Server with static IP
//******************************************************************************
char WiFi2_TCPS_STATIP() {
  WiFi2_SetCommandMode();
  WiFi2_CMD(&AT_DEFAULT, "1");                                                  //Set module to default
  while(!WaitToResponse(&AT_DEFAULT));
  delay_ms(500);
  //User configuration
  WiFi2_CMD(&AT_WM, "2");                                                       //Select manual STA work method
  while(!WaitToResponse(&AT_WM));
  WiFi2_CMD(&AT_IP, &myIP);                                                     //Set static IP
  while(!WaitToResponse(&AT_IP));
  WiFi2_CMD(&AT_MASK, &gwMASK);                                                 //Set gateway mask
  while(!WaitToResponse(&AT_MASK));
  WiFi2_CMD(&AT_GW, &gwIP);                                                     //Set gateway IP
  while(!WaitToResponse(&AT_GW));
  WiFi2_CMD(&AT_DNS, &gwDNS);                                                   //Set gateway DNS
  while(!WaitToResponse(&AT_DNS));
  WiFi2_CMD(&AT_SSSID, &gwSSID);                                                //Set gateway SSID
  while(!WaitToResponse(&AT_SSSID));
  WiFi2_CMD(&AT_SSSIDL, &gwSSIDL);                                              //Set lenght of gateway SSID
  while(!WaitToResponse(&AT_SSSIDL));
  WiFi2_CMD(&AT_SAM, "4");                                                      //Set gateway encryption method to AES
  while(!WaitToResponse(&AT_SAM));
  WiFi2_CMD(&AT_SPW, &gwPW);                                                    //Set gateway key
  while(!WaitToResponse(&AT_SPW));
  WiFi2_CMD(&AT_SPWL, &gwPWL);                                                  //Set lenght of gateway key
  while(!WaitToResponse(&AT_SPWL));
  WiFi2_CMD(&AT_DHCP, "0");                                                     //Configure module with static IP
  while(!WaitToResponse(&AT_DHCP));
  WiFi2_CMD(&AT_UTYPE, "1");                                                    //Set TCP
  while(!WaitToResponse(&AT_UTYPE));
  WiFi2_CMD(&AT_ULPORT, &myPORT);                                                //Set local port
  while(!WaitToResponse(&AT_ULPORT));
  WiFi2_CMD(&AT_WC, "1");                                                       //Calculation PMK
  while(!WaitToResponse(&AT_WC));
  WiFi2_CMD(&AT_REBOOT, "1");                                                   //Reboot module
 while(!WaitToResponse(&AT_REBOOT));
  return 1;
}

//******************************************************************************
//Configure WiFi2 click as TCP Server with dynamic IP
//DHCP enabled
//******************************************************************************
char WiFi2_TCPS_DYNIP() {
  WiFi2_SetCommandMode();
  WiFi2_CMD(&AT_DEFAULT, "1");                                                  //Set module to default
  while(!WaitToResponse(&AT_DEFAULT));
  delay_ms(500);
  //User configuration
  WiFi2_CMD(&AT_WM, "2");                                                       //Select manual STA work method
  while(!WaitToResponse(&AT_WM));
  WiFi2_CMD(&AT_SSSID, &gwSSID);                                                //Set gateway SSID
  while(!WaitToResponse(&AT_SSSID));
  WiFi2_CMD(&AT_SSSIDL, &gwSSIDL);                                              //Set lenght of gateway SSID
  while(!WaitToResponse(&AT_SSSIDL));
  WiFi2_CMD(&AT_SAM, "4");                                                      //Set gateway encryption method to AES
  while(!WaitToResponse(&AT_SAM));
  WiFi2_CMD(&AT_SPW, &gwPW);                                                    //Set gateway key
  while(!WaitToResponse(&AT_SPW));
  WiFi2_CMD(&AT_SPWL, &gwPWL);                                                  //Set lenght of gateway key
  while(!WaitToResponse(&AT_SPWL));
  WiFi2_CMD(&AT_DHCP, "1");                                                     //Configure module with dynamic IP
  while(!WaitToResponse(&AT_DHCP));
  WiFi2_CMD(&AT_UTYPE, "1");                                                    //Set TCP
  while(!WaitToResponse(&AT_UTYPE));
  WiFi2_CMD(&AT_ULPORT, &myPORT);                                               //Set local port
  while(!WaitToResponse(&AT_ULPORT));
  WiFi2_CMD(&AT_WC, "1");                                                       //Calculation PMK
  while(!WaitToResponse(&AT_WC));
  WiFi2_CMD(&AT_REBOOT, "1");                                                   //Reboot module
  while(!WaitToResponse(&AT_REBOOT));
  return 1;
}

//******************************************************************************
//Configure WiFi2 click as UDP Server with static IP
//******************************************************************************
char WiFi2_UDPS_STATIP() {
  WiFi2_SetCommandMode();
  WiFi2_CMD(&AT_DEFAULT, "1");                                                  //Set module to default
  while(!WaitToResponse(&AT_DEFAULT));
  delay_ms(500);
  //User configuration
  WiFi2_CMD(&AT_WM, "2");                                                       //Select manual STA work method
  while(!WaitToResponse(&AT_WM));
  WiFi2_CMD(&AT_IP, &myIP);                                                     //Set static IP
  while(!WaitToResponse(&AT_IP));
  WiFi2_CMD(&AT_MASK, &gwMASK);                                                 //Set gateway mask
  while(!WaitToResponse(&AT_MASK));
  WiFi2_CMD(&AT_GW, &gwIP);                                                     //Set gateway IP
  while(!WaitToResponse(&AT_GW));
  WiFi2_CMD(&AT_DNS, &gwDNS);                                                   //Set gateway DNS
  while(!WaitToResponse(&AT_DNS));
  WiFi2_CMD(&AT_SSSID, &gwSSID);                                                //Set gateway SSID
  while(!WaitToResponse(&AT_SSSID));
  WiFi2_CMD(&AT_SSSIDL, &gwSSIDL);                                              //Set lenght of gateway SSID
  while(!WaitToResponse(&AT_SSSIDL));
  WiFi2_CMD(&AT_SAM, "4");                                                      //Set gateway encryption method to AES
  while(!WaitToResponse(&AT_SAM));
  WiFi2_CMD(&AT_SPW, &gwPW);                                                    //Set gateway key
  while(!WaitToResponse(&AT_SPW));
  WiFi2_CMD(&AT_SPWL, &gwPWL);                                                  //Set lenght of gateway key
  while(!WaitToResponse(&AT_SPWL));
  WiFi2_CMD(&AT_DHCP, "0");                                                     //Configure module with static IP
  while(!WaitToResponse(&AT_DHCP));
  WiFi2_CMD(&AT_UTYPE, "3");                                                    //Set UDP Server
  while(!WaitToResponse(&AT_UTYPE));
  WiFi2_CMD(&AT_ULPORT, &myPORT);                                               //Set local port
  while(!WaitToResponse(&AT_ULPORT));
  WiFi2_CMD(&AT_WC, "1");                                                       //Calculation PMK
  while(!WaitToResponse(&AT_WC));
  WiFi2_CMD(&AT_REBOOT, "1");                                                   //Reboot module
 while(!WaitToResponse(&AT_REBOOT));
  return 1;
}


//******************************************************************************
//Configure WiFi2 click as UDP Server with dynamic IP
//DHCP enabled
//******************************************************************************
char WiFi2_UDPS_DYNIP() {
  WiFi2_SetCommandMode();
  WiFi2_CMD(&AT_DEFAULT, "1");                                                  //Set module to default
  while(!WaitToResponse(&AT_DEFAULT));
  delay_ms(500);
  //User configuration
  WiFi2_CMD(&AT_WM, "2");                                                       //Select manual STA work method
  while(!WaitToResponse(&AT_WM));
  WiFi2_CMD(&AT_SSSID, &gwSSID);                                                //Set gateway SSID
  while(!WaitToResponse(&AT_SSSID));
  WiFi2_CMD(&AT_SSSIDL, &gwSSIDL);                                              //Set lenght of gateway SSID
  while(!WaitToResponse(&AT_SSSIDL));
  WiFi2_CMD(&AT_SAM, "4");                                                      //Set gateway encryption method to AES
  while(!WaitToResponse(&AT_SAM));
  WiFi2_CMD(&AT_SPW, &gwPW);                                                    //Set gateway key
  while(!WaitToResponse(&AT_SPW));
  WiFi2_CMD(&AT_SPWL, &gwPWL);                                                  //Set lenght of gateway key
  while(!WaitToResponse(&AT_SPWL));
  WiFi2_CMD(&AT_DHCP, "1");                                                     //Configure module with dynamic IP
  while(!WaitToResponse(&AT_DHCP));
  WiFi2_CMD(&AT_UTYPE, "3");                                                    //Set UDP Server
  while(!WaitToResponse(&AT_UTYPE));
  WiFi2_CMD(&AT_ULPORT, &myPORT);                                                //Set local port
  while(!WaitToResponse(&AT_ULPORT));
  WiFi2_CMD(&AT_WC, "1");                                                       //Calculation PMK
  while(!WaitToResponse(&AT_WC));
  WiFi2_CMD(&AT_REBOOT, "1");                                                   //Reboot module
  while(!WaitToResponse(&AT_REBOOT));
  return 1;
}


//******************************************************************************
//                             CLIENT CONFIG
//******************************************************************************


//******************************************************************************
//Configure WiFi2 click as TCP Client with static IP
//******************************************************************************
char WiFi2_TCPC_STATIP() {
  WiFi2_SetCommandMode();
  WiFi2_CMD(&AT_DEFAULT, "1");                                                  //Set module to default
  while(!WaitToResponse(&AT_DEFAULT));
  delay_ms(500);
  //User configuration
  WiFi2_CMD(&AT_WM, "2");                                                       //Select manual STA work method
  while(!WaitToResponse(&AT_WM));
  WiFi2_CMD(&AT_IP, &myIP);                                                     //Set static IP
  while(!WaitToResponse(&AT_IP));
  WiFi2_CMD(&AT_MASK, &gwMASK);                                                 //Set gateway mask
  while(!WaitToResponse(&AT_MASK));
  WiFi2_CMD(&AT_GW, &gwIP);                                                     //Set gateway IP
  while(!WaitToResponse(&AT_GW));
  WiFi2_CMD(&AT_DNS, &gwDNS);                                                   //Set gateway DNS
  while(!WaitToResponse(&AT_DNS));
  WiFi2_CMD(&AT_SSSID, &gwSSID);                                                //Set gateway SSID
  while(!WaitToResponse(&AT_SSSID));
  WiFi2_CMD(&AT_SSSIDL, &gwSSIDL);                                              //Set lenght of gateway SSID
  while(!WaitToResponse(&AT_SSSIDL));
  WiFi2_CMD(&AT_SAM, "4");                                                      //Set gateway encryption method to AES
  while(!WaitToResponse(&AT_SAM));
  WiFi2_CMD(&AT_SPW, &gwPW);                                                    //Set gateway key
  while(!WaitToResponse(&AT_SPW));
  WiFi2_CMD(&AT_SPWL, &gwPWL);                                                  //Set lenght of gateway key
  while(!WaitToResponse(&AT_SPWL));
  WiFi2_CMD(&AT_DHCP, "0");                                                     //Configure module with static IP
  while(!WaitToResponse(&AT_DHCP));
  WiFi2_CMD(&AT_UTYPE, "2");                                                    //Set TCP Client
  while(!WaitToResponse(&AT_UTYPE));
  WiFi2_CMD(&AT_ULPORT, &myPORT);                                               //Set local port
  while(!WaitToResponse(&AT_ULPORT));
  WiFi2_CMD(&AT_URPORT, &rmPORT);                                               //Set remote port
  while(!WaitToResponse(&AT_URPORT));
  WiFi2_CMD(&AT_UIP, &rmIP);                                                    //Set remote IP
  while(!WaitToResponse(&AT_UIP));
  WiFi2_CMD(&AT_WC, "1");                                                       //Calculation PMK
  while(!WaitToResponse(&AT_WC));
  WiFi2_CMD(&AT_REBOOT, "1");                                                   //Reboot module
 while(!WaitToResponse(&AT_REBOOT));
  return 1;
}

//******************************************************************************
//Configure WiFi2 click as TCP Client with dynamic IP
//DHCP enabled
//******************************************************************************
char WiFi2_TCPC_DYNIP() {
  WiFi2_SetCommandMode();
  WiFi2_CMD(&AT_DEFAULT, "1");                                                  //Set module to default
  while(!WaitToResponse(&AT_DEFAULT));
  delay_ms(500);
  //User configuration
  WiFi2_CMD(&AT_WM, "2");                                                       //Select manual STA work method
  while(!WaitToResponse(&AT_WM));
  WiFi2_CMD(&AT_SSSID, &gwSSID);                                                //Set gateway SSID
  while(!WaitToResponse(&AT_SSSID));
  WiFi2_CMD(&AT_SSSIDL, &gwSSIDL);                                              //Set lenght of gateway SSID
  while(!WaitToResponse(&AT_SSSIDL));
  WiFi2_CMD(&AT_SAM, "4");                                                      //Set gateway encryption method to AES
  while(!WaitToResponse(&AT_SAM));
  WiFi2_CMD(&AT_SPW, &gwPW);                                                    //Set gateway key
  while(!WaitToResponse(&AT_SPW));
  WiFi2_CMD(&AT_SPWL, &gwPWL);                                                  //Set lenght of gateway key
  while(!WaitToResponse(&AT_SPWL));
  WiFi2_CMD(&AT_DHCP, "1");                                                     //Configure module with dynamic IP
  while(!WaitToResponse(&AT_DHCP));
  WiFi2_CMD(&AT_UTYPE, "2");                                                    //Set TCP Client
  while(!WaitToResponse(&AT_UTYPE));
  WiFi2_CMD(&AT_ULPORT, &myPORT);                                                //Set local port
  while(!WaitToResponse(&AT_ULPORT));
  WiFi2_CMD(&AT_URPORT, &rmPORT);                                               //Set remote port
  while(!WaitToResponse(&AT_URPORT));
  WiFi2_CMD(&AT_UIP, &rmIP);                                                    //Set remote IP
  while(!WaitToResponse(&AT_UIP));
  WiFi2_CMD(&AT_WC, "1");                                                       //Calculation PMK
  while(!WaitToResponse(&AT_WC));
  WiFi2_CMD(&AT_REBOOT, "1");                                                   //Reboot module
  while(!WaitToResponse(&AT_REBOOT));
  return 1;
}

//******************************************************************************
//Configure WiFi2 click as UDP Client with static IP
//******************************************************************************
char WiFi2_UDPC_STATIP() {
  WiFi2_SetCommandMode();
  WiFi2_CMD(&AT_DEFAULT, "1");                                                  //Set module to default
  while(!WaitToResponse(&AT_DEFAULT));
  delay_ms(500);
  //User configuration
  WiFi2_CMD(&AT_WM, "2");                                                       //Select manual STA work method
  while(!WaitToResponse(&AT_WM));
  WiFi2_CMD(&AT_IP, &myIP);                                                     //Set static IP
  while(!WaitToResponse(&AT_IP));
  WiFi2_CMD(&AT_MASK, &gwMASK);                                                 //Set gateway mask
  while(!WaitToResponse(&AT_MASK));
  WiFi2_CMD(&AT_GW, &gwIP);                                                     //Set gateway IP
  while(!WaitToResponse(&AT_GW));
  WiFi2_CMD(&AT_DNS, &gwDNS);                                                   //Set gateway DNS
  while(!WaitToResponse(&AT_DNS));
  WiFi2_CMD(&AT_SSSID, &gwSSID);                                                //Set gateway SSID
  while(!WaitToResponse(&AT_SSSID));
  WiFi2_CMD(&AT_SSSIDL, &gwSSIDL);                                              //Set lenght of gateway SSID
  while(!WaitToResponse(&AT_SSSIDL));
  WiFi2_CMD(&AT_SAM, "4");                                                      //Set gateway encryption method to AES
  while(!WaitToResponse(&AT_SAM));
  WiFi2_CMD(&AT_SPW, &gwPW);                                                    //Set gateway key
  while(!WaitToResponse(&AT_SPW));
  WiFi2_CMD(&AT_SPWL, &gwPWL);                                                  //Set lenght of gateway key
  while(!WaitToResponse(&AT_SPWL));
  WiFi2_CMD(&AT_DHCP, "0");                                                     //Configure module with static IP
  while(!WaitToResponse(&AT_DHCP));
  WiFi2_CMD(&AT_UTYPE, "4");                                                    //Set UDP Client
  while(!WaitToResponse(&AT_UTYPE));
  WiFi2_CMD(&AT_ULPORT, &myPORT);                                               //Set local port
  while(!WaitToResponse(&AT_ULPORT));
  WiFi2_CMD(&AT_URPORT, &rmPORT);                                               //Set remote port
  while(!WaitToResponse(&AT_URPORT));
  WiFi2_CMD(&AT_UIP, &rmIP);                                                    //Set remote IP
  while(!WaitToResponse(&AT_UIP));
  WiFi2_CMD(&AT_WC, "1");                                                       //Calculation PMK
  while(!WaitToResponse(&AT_WC));
  WiFi2_CMD(&AT_REBOOT, "1");                                                   //Reboot module
 while(!WaitToResponse(&AT_REBOOT));
  return 1;
}


//******************************************************************************
//Configure WiFi2 click as UDP Client with dynamic IP
//DHCP enabled
//******************************************************************************
char WiFi2_UDPC_DYNIP() {
  WiFi2_SetCommandMode();
  WiFi2_CMD(&AT_DEFAULT, "1");                                                  //Set module to default
  while(!WaitToResponse(&AT_DEFAULT));
  delay_ms(500);
  //User configuration
  WiFi2_CMD(&AT_WM, "2");                                                       //Select manual STA work method
  while(!WaitToResponse(&AT_WM));
  WiFi2_CMD(&AT_SSSID, &gwSSID);                                                //Set gateway SSID
  while(!WaitToResponse(&AT_SSSID));
  WiFi2_CMD(&AT_SSSIDL, &gwSSIDL);                                              //Set lenght of gateway SSID
  while(!WaitToResponse(&AT_SSSIDL));
  WiFi2_CMD(&AT_SAM, "4");                                                      //Set gateway encryption method to AES
  while(!WaitToResponse(&AT_SAM));
  WiFi2_CMD(&AT_SPW, &gwPW);                                                    //Set gateway key
  while(!WaitToResponse(&AT_SPW));
  WiFi2_CMD(&AT_SPWL, &gwPWL);                                                  //Set lenght of gateway key
  while(!WaitToResponse(&AT_SPWL));
  WiFi2_CMD(&AT_DHCP, "1");                                                     //Configure module with dynamic IP
  while(!WaitToResponse(&AT_DHCP));
  WiFi2_CMD(&AT_UTYPE, "4");                                                    //Set UDP Client
  while(!WaitToResponse(&AT_UTYPE));
  WiFi2_CMD(&AT_ULPORT, &myPORT);                                               //Set local port
  while(!WaitToResponse(&AT_ULPORT));
  WiFi2_CMD(&AT_URPORT, &rmPORT);                                               //Set remote port
  while(!WaitToResponse(&AT_URPORT));
  WiFi2_CMD(&AT_UIP, &rmIP);                                                    //Set remote IP
  while(!WaitToResponse(&AT_UIP));
  WiFi2_CMD(&AT_WC, "1");                                                       //Calculation PMK
  while(!WaitToResponse(&AT_WC));
  WiFi2_CMD(&AT_REBOOT, "1");                                                   //Reboot module
  while(!WaitToResponse(&AT_REBOOT));
  return 1;
}