/*
 * Project name:
     Wifi2 Click Board Example
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20140225:
       - initial release (FJ);
 * Description:
     This is a simple demonstration of WiFi Click Board usage.
 * Test configuration:
     MCU:             STM32F107VC
                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/reference_manual/CD00171190.pdf
     Dev. Board:      EasyMx PRO v7 for STM32
                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
     Oscillator:      HS-PLL 72.0000 MHz
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 * NOTES:
     - Place WiFi Click board at the mikroBUS socket 2 on the EasyMx Pro v7 board.
     - To configure your WiFi2 Click use WiFi2_Config.h
     - Use TCP terminal to communicate with WiFi2 Click board
  */
  
#include "HLK_M30_Driver.h"

// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

extern sfr sbit WiFi2_DEF;
extern sfr sbit WiFi2_RST;
extern sfr sbit WiFi2_TX;
extern sfr sbit WiFi2_RX;

extern char rx_buff[64];
extern char data_ready;
extern char data_len;

//USER COMMANDS
const char RED[]   = "RED";     // Fill TFT screen with red colour
const char BLUE[]  = "BLUE";    // Fill TFT screen with blue colour
const char GREEN[] = "GREEN";   // Fill TFT screen with green colour

char tmp;
char err;
unsigned int cnt = 0;

void UART2_Interrupt() iv IVT_INT_USART2 ics ICS_AUTO {
  tmp = UART2_Read();
  if (tmp == 0x0D) {
    rx_buff[data_len + 1] = 0;
    data_ready = 1;
  }
  else {
    rx_buff[data_len] = tmp;
    data_len++;
  }
}

void Display_Init() {
  TFT_Init_ILI9341_8bit(320, 240);
  TFT_BLED = 1;
}

void DrawFrame() {
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_BLACK, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20, 46, 300, 46);
  TFT_Set_Font(&TFT_defaultFont, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("WiFi2  Click  Board  Demo", 25, 14);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyMx PRO v7", 19, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLUE, FO_HORIZONTAL);
}

void updateMe(char pos){
  TFT_Write_Char('_', 30 + pos, 61);
}

void main() {
  WiFi2_Init();
  Display_Init();
  DrawFrame();
  
  RXNEIE_USART2_CR1_bit = 1;
  NVIC_IntEnable(IVT_INT_USART2);
  
  //Config module
  TFT_Write_Text("Initialization...", 40, 60);
  while (WiFi2_TCPS_STATIP() == 0);

  TFT_Write_Text("Initialization...Done", 40, 60);
  TFT_Write_Text("TCP Server Listening...", 40, 80);
  while (1) {
    if (data_ready && data_len) {
      if (strstr(rx_buff, RED)) {
        TFT_Set_Font(&TFT_defaultFont, CL_RED, FO_HORIZONTAL);
        UART2_Write(13);
        UART2_Write_Text("TFT Font is RED");
        UART2_Write(13);
      } else if (strstr(rx_buff, BLUE)) {
        TFT_Set_Font(&TFT_defaultFont, CL_BLUE, FO_HORIZONTAL);
        UART2_Write(13);
        UART2_Write_Text("TFT Font is BLUE");
        UART2_Write(13);
      } else if (strstr(rx_buff, GREEN)) {
        TFT_Set_Font(&TFT_defaultFont, CL_GREEN, FO_HORIZONTAL);
        UART2_Write(13);
        UART2_Write_Text("TFT Font is GREEN");
        UART2_Write(13);
      } else {
        TFT_Set_Pen(CL_WHITE, 10);
        TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);
        TFT_Rectangle(20, 120, 300, 200);
        TFT_Write_Text(rx_buff, 40, 120);
      }
      data_ready = 0;
      memset(rx_buff, 0, data_len);
      data_len = 0;
    }
  }
}